/**
 *******************************************************************************
 * @file    spi.c
 *
 * @brief   spi program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/
#include "spi.h"

/**
  * @brief  This function is SPI GPIO Init.
  * @param  None
  * @retval None
  */
void SPI_GPIO_Init(void)
{
  GPIO_InitTypeStruct GPIO_InitStruct;

  /* Config GPIO */
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_ALT_FUNC;
  GPIO_InitStruct.GPIO_Speed = GPIO_SPEED_HIGH;
  GPIO_InitStruct.GPIO_OType = GPIO_OTYPE_PP;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_NOPULL;
  //SCLK
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_13;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOB, GPIO_PIN_13, GPIO_ALT_FUNC_0);
  //MISO
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_14;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOB, GPIO_PIN_14, GPIO_ALT_FUNC_0);
  //MOSI
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_15;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
  GPIO_AltFuncConfig(GPIOB, GPIO_PIN_15, GPIO_ALT_FUNC_0);
  //CS
  GPIO_InitStruct.GPIO_Mode = GPIO_MODE_OUTPUT;
  GPIO_InitStruct.GPIO_PuPd = GPIO_PUPD_UP;
  GPIO_InitStruct.GPIO_Pin = GPIO_PIN_12;
  GPIO_Init(GPIOB, &GPIO_InitStruct);
}

/**
  * @brief  This function is SPI Config.
  * @param  None
  * @retval None
  */
void SPI_Config(void)
{
  /* Enable clock */
  RCC_Unlock();
  RCC_APBPeriphClockCmd(RCC_APB_PERIPH_SPI1, ENABLE);
  RCC_Lock();

  /* Config SPI0 */
  SPI_InitTypeStruct SPI_InitStruct;

  SPI_InitStruct.SPI_Mode = SPI_MODE_MASTER;
  SPI_InitStruct.SPI_DataSize = SPI_DATASIZE_8BIT;
  SPI_InitStruct.SPI_CPOL = SPI_CPOL_LOW;
  SPI_InitStruct.SPI_CPHA = SPI_CPHA_1EDGE;
  SPI_InitStruct.SPI_BaudRatePrescaler = SPI_BAUDRATE_PRESCALER_128;
  SPI_InitStruct.SPI_FirstBit = SPI_FRIST_BIT_MSB;
  SPI_InitStruct.SPI_NSSSlectEn = SPI_NSS_SELECT_DISABLE;
  SPI_Init(SPI1, &SPI_InitStruct);
  /* Enable SPI1) */
  SPI_Cmd(SPI1, ENABLE);
}

/**
  * @brief  This function is SPI Trasmit.
  * @param  None
  * @retval None
  */
uint8_t SPI_TransmitReceive(SPI_Type *SPIx, uint8_t txData)
{
  while(SPI_GetIntFlagStatus(SPIx, SPI_INT_FLAG_TXE) != SET);
  SPI_SendData(SPIx, txData);

  while(SPI_GetIntFlagStatus(SPIx, SPI_INT_FLAG_RXNE) == RESET);
  return SPI_ReceiveData(SPIx);
}
